@@ -1,6 +1,6 @@
 use std::collections::{HashMap, HashSet};
 
-use failure::bail;
+use anyhow::bail;
 use log::{debug, trace};
 use semver::VersionReq;
 use url::Url;
@@ -255,7 +255,7 @@ impl<'cfg> PackageRegistry<'cfg> {
                 // corresponding to this `dep`.
                 self.ensure_loaded(dep.source_id(), Kind::Normal)
                     .chain_err(|| {
-                        failure::format_err!(
+                        anyhow::format_err!(
                             "failed to load source for a dependency \
                              on `{}`",
                             dep.package_name()
@@ -271,7 +271,7 @@ impl<'cfg> PackageRegistry<'cfg> {
 
                 let summary = match summaries.next() {
                     Some(summary) => summary,
-                    None => failure::bail!(
+                    None => anyhow::bail!(
                         "patch for `{}` in `{}` did not resolve to any crates. If this is \
                          unexpected, you may wish to consult: \
                          https://github.com/rust-lang/cargo/issues/4678",
@@ -280,14 +280,14 @@ impl<'cfg> PackageRegistry<'cfg> {
                     ),
                 };
                 if summaries.next().is_some() {
-                    failure::bail!(
+                    anyhow::bail!(
                         "patch for `{}` in `{}` resolved to more than one candidate",
                         dep.package_name(),
                         url
                     )
                 }
                 if *summary.package_id().source_id().canonical_url() == canonical {
-                    failure::bail!(
+                    anyhow::bail!(
                         "patch for `{}` in `{}` points to the same source, but \
                          patches must point to different sources",
                         dep.package_name(),
@@ -297,7 +297,7 @@ impl<'cfg> PackageRegistry<'cfg> {
                 Ok(summary)
             })
             .collect::<CargoResult<Vec<_>>>()
-            .chain_err(|| failure::format_err!("failed to resolve patches for `{}`", url))?;
+            .chain_err(|| anyhow::format_err!("failed to resolve patches for `{}`", url))?;
 
         let mut name_and_version = HashSet::new();
         for summary in unlocked_summaries.iter() {
@@ -364,7 +364,7 @@ impl<'cfg> PackageRegistry<'cfg> {
             let _p = profile::start(format!("updating: {}", source_id));
             self.sources.get_mut(source_id).unwrap().update()
         })()
-        .chain_err(|| failure::format_err!("Unable to update {}", source_id))?;
+        .chain_err(|| anyhow::format_err!("Unable to update {}", source_id))?;
         Ok(())
     }
 
@@ -419,7 +419,7 @@ To change the dependency graph via an override it's recommended to use the
 `[replace]` feature of Cargo instead of the path override feature. This is
 documented online at the url below for more information.
 
-https://doc.rust-lang.org/cargo/reference/specifying-dependencies.html#overriding-dependencies
+https://doc.rust-lang.org/cargo/reference/overriding-dependencies.html
 ";
 
         for dep in override_summary.dependencies() {
@@ -516,7 +516,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                 // Ensure the requested source_id is loaded
                 self.ensure_loaded(dep.source_id(), Kind::Normal)
                     .chain_err(|| {
-                        failure::format_err!(
+                        anyhow::format_err!(
                             "failed to load source for a dependency \
                              on `{}`",
                             dep.package_name()
@@ -525,7 +525,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
 
                 let source = self.sources.get_mut(dep.source_id());
                 match (override_summary, source) {
-                    (Some(_), None) => failure::bail!("override found but no real ones"),
+                    (Some(_), None) => anyhow::bail!("override found but no real ones"),
                     (None, None) => return Ok(()),
 
                     // If we don't have an override then we just ship
@@ -565,7 +565,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
                     // the summaries it gives us though.
                     (Some(override_summary), Some(source)) => {
                         if !patches.is_empty() {
-                            failure::bail!("found patches and a path override")
+                            anyhow::bail!("found patches and a path override")
                         }
                         let mut n = 0;
                         let mut to_warn = None;
@@ -587,7 +587,7 @@ impl<'cfg> Registry for PackageRegistry<'cfg> {
         };
 
         if n > 1 {
-            failure::bail!("found an override with a non-locked list");
+            anyhow::bail!("found an override with a non-locked list");
         } else if let Some(summary) = to_warn {
             self.warn_bad_override(&override_summary, &summary)?;
         }
