@@ -30,15 +30,12 @@
 // https://github.com/rust-lang/cargo/pull/7251#pullrequestreview-274914270
 #![allow(clippy::identity_conversion)]
 
-use std::fmt;
-use std::io;
-
-use failure::Error;
-use log::debug;
-use serde::ser;
-
 use crate::core::shell::Verbosity::Verbose;
 use crate::core::Shell;
+use anyhow::Error;
+use log::debug;
+use serde::ser;
+use std::fmt;
 
 pub use crate::util::errors::Internal;
 pub use crate::util::{CargoResult, CliError, CliResult, Config};
@@ -140,7 +137,7 @@ pub fn exit_with_error(err: CliError, shell: &mut Shell) -> ! {
     std::process::exit(exit_code)
 }
 
-pub fn handle_error(err: &failure::Error, shell: &mut Shell) {
+pub fn handle_error(err: &Error, shell: &mut Shell) {
     debug!("handle_error; err={:?}", err);
 
     let _ignored_result = shell.error(err);
@@ -153,18 +150,10 @@ fn handle_cause(cargo_err: &Error, shell: &mut Shell) -> bool {
         drop(writeln!(shell.err(), "  {}", error));
     }
 
-    fn print_stderror_causes(error: &dyn std::error::Error, shell: &mut Shell) {
-        let mut cur = std::error::Error::source(error);
-        while let Some(err) = cur {
-            print(&err.to_string(), shell);
-            cur = std::error::Error::source(err);
-        }
-    }
-
     let verbose = shell.verbosity();
 
     // The first error has already been printed to the shell.
-    for err in cargo_err.iter_causes() {
+    for err in cargo_err.chain().skip(1) {
         // If we're not in verbose mode then print remaining errors until one
         // marked as `Internal` appears.
         if verbose != Verbose && err.downcast_ref::<Internal>().is_some() {
@@ -172,19 +161,6 @@ fn handle_cause(cargo_err: &Error, shell: &mut Shell) -> bool {
         }
 
         print(&err.to_string(), shell);
-
-        // Using the `failure` crate currently means that when using
-        // `iter_causes` we're only iterating over the `failure` causes, but
-        // this doesn't include the causes from the standard library `Error`
-        // trait. We don't have a great way of getting an `&dyn Error` from a
-        // `&dyn Fail`, so we currently just special case a few errors that are
-        // known to maybe have causes and we try to print them here.
-        //
-        // Note that this isn't an exhaustive match since causes for
-        // `std::error::Error` aren't the most common thing in the world.
-        if let Some(io) = err.downcast_ref::<io::Error>() {
-            print_stderror_causes(io, shell);
-        }
     }
 
     true
