@@ -1,5 +1,5 @@
 use crate::core::{compiler, Workspace};
-use crate::util::errors::{self, CargoResult, CargoResultExt};
+use crate::util::errors::{CargoResult, CargoResultExt};
 use crate::util::{existing_vcs_repo, FossilRepo, GitRepo, HgRepo, PijulRepo};
 use crate::util::{paths, validate_package_name, Config};
 use git2::Config as GitConfig;
@@ -26,16 +26,16 @@ pub enum VersionControl {
 }
 
 impl FromStr for VersionControl {
-    type Err = failure::Error;
+    type Err = anyhow::Error;
 
-    fn from_str(s: &str) -> Result<Self, failure::Error> {
+    fn from_str(s: &str) -> Result<Self, anyhow::Error> {
         match s {
             "git" => Ok(VersionControl::Git),
             "hg" => Ok(VersionControl::Hg),
             "pijul" => Ok(VersionControl::Pijul),
             "fossil" => Ok(VersionControl::Fossil),
             "none" => Ok(VersionControl::NoVcs),
-            other => failure::bail!("unknown vcs specification: `{}`", other),
+            other => anyhow::bail!("unknown vcs specification: `{}`", other),
         }
     }
 }
@@ -110,7 +110,7 @@ impl NewOptions {
         registry: Option<String>,
     ) -> CargoResult<NewOptions> {
         let kind = match (bin, lib) {
-            (true, true) => failure::bail!("can't specify both lib and binary outputs"),
+            (true, true) => anyhow::bail!("can't specify both lib and binary outputs"),
             (false, true) => NewProjectKind::Lib,
             // default to bin
             (_, false) => NewProjectKind::Bin,
@@ -142,14 +142,14 @@ fn get_name<'a>(path: &'a Path, opts: &'a NewOptions) -> CargoResult<&'a str> {
     }
 
     let file_name = path.file_name().ok_or_else(|| {
-        failure::format_err!(
+        anyhow::format_err!(
             "cannot auto-detect package name from path {:?} ; use --name to override",
             path.as_os_str()
         )
     })?;
 
     file_name.to_str().ok_or_else(|| {
-        failure::format_err!(
+        anyhow::format_err!(
             "cannot create package with a non-unicode name: {:?}",
             file_name
         )
@@ -174,16 +174,16 @@ fn check_name(name: &str, opts: &NewOptions) -> CargoResult<()> {
         "true", "type", "typeof", "unsafe", "unsized", "use", "virtual", "where", "while", "yield",
     ];
     if blacklist.contains(&name) || (opts.kind.is_bin() && compiler::is_bad_artifact_name(name)) {
-        failure::bail!(
+        anyhow::bail!(
             "The name `{}` cannot be used as a crate name{}",
             name,
             name_help
         )
     }
 
-    if let Some(ref c) = name.chars().nth(0) {
+    if let Some(ref c) = name.chars().next() {
         if c.is_digit(10) {
-            failure::bail!(
+            anyhow::bail!(
                 "Package names starting with a digit cannot be used as a crate name{}",
                 name_help
             )
@@ -283,7 +283,7 @@ fn detect_source_paths_and_types(
     for i in detected_files {
         if i.bin {
             if let Some(x) = BTreeMap::get::<str>(&duplicates_checker, i.target_name.as_ref()) {
-                failure::bail!(
+                anyhow::bail!(
                     "\
 multiple possible binary sources found:
   {}
@@ -296,7 +296,7 @@ cannot automatically generate Cargo.toml as the main target would be ambiguous",
             duplicates_checker.insert(i.target_name.as_ref(), i);
         } else {
             if let Some(plp) = previous_lib_relpath {
-                failure::bail!(
+                anyhow::bail!(
                     "cannot have a package with \
                      multiple libraries, \
                      found both `{}` and `{}`",
@@ -330,7 +330,7 @@ fn plan_new_source_file(bin: bool, package_name: String) -> SourceFileInformatio
 pub fn new(opts: &NewOptions, config: &Config) -> CargoResult<()> {
     let path = &opts.path;
     if fs::metadata(path).is_ok() {
-        failure::bail!(
+        anyhow::bail!(
             "destination `{}` already exists\n\n\
              Use `cargo init` to initialize the directory",
             path.display()
@@ -351,7 +351,7 @@ pub fn new(opts: &NewOptions, config: &Config) -> CargoResult<()> {
     };
 
     mk(config, &mkopts).chain_err(|| {
-        failure::format_err!(
+        anyhow::format_err!(
             "Failed to create package `{}` at `{}`",
             name,
             path.display()
@@ -364,7 +364,7 @@ pub fn init(opts: &NewOptions, config: &Config) -> CargoResult<()> {
     let path = &opts.path;
 
     if fs::metadata(&path.join("Cargo.toml")).is_ok() {
-        failure::bail!("`cargo init` cannot be run on existing Cargo packages")
+        anyhow::bail!("`cargo init` cannot be run on existing Cargo packages")
     }
 
     let name = get_name(path, opts)?;
@@ -410,7 +410,7 @@ pub fn init(opts: &NewOptions, config: &Config) -> CargoResult<()> {
         // if none exists, maybe create git, like in `cargo new`
 
         if num_detected_vsces > 1 {
-            failure::bail!(
+            anyhow::bail!(
                 "more than one of .hg, .git, .pijul, .fossil configurations \
                  found and the ignore file can't be filled in as \
                  a result. specify --vcs to override detection"
@@ -429,7 +429,7 @@ pub fn init(opts: &NewOptions, config: &Config) -> CargoResult<()> {
     };
 
     mk(config, &mkopts).chain_err(|| {
-        failure::format_err!(
+        anyhow::format_err!(
             "Failed to create package `{}` at `{}`",
             name,
             path.display()
@@ -527,7 +527,7 @@ fn write_ignore_file(
     let ignore: String = match fs::File::open(&fp_ignore) {
         Err(why) => match why.kind() {
             ErrorKind::NotFound => list.format_new(vcs),
-            _ => return Err(failure::format_err!("{}", why)),
+            _ => return Err(anyhow::format_err!("{}", why)),
         },
         Ok(file) => list.format_existing(BufReader::new(file), vcs),
     };
@@ -713,8 +713,8 @@ mod tests {
     if let Err(e) = Workspace::new(&path.join("Cargo.toml"), config) {
         let msg = format!(
             "compiling this new crate may not work due to invalid \
-             workspace configuration\n\n{}",
-            errors::display_causes(&e)
+             workspace configuration\n\n{:?}",
+            e,
         );
         config.shell().warn(msg)?;
     }
@@ -752,7 +752,7 @@ fn discover_author() -> CargoResult<(String, Option<String>)> {
         Some(name) => name,
         None => {
             let username_var = if cfg!(windows) { "USERNAME" } else { "USER" };
-            failure::bail!(
+            anyhow::bail!(
                 "could not determine the current user, please set ${}",
                 username_var
             )
