@@ -54,7 +54,7 @@ impl Rustc {
                 .find(|l| l.starts_with("host: "))
                 .map(|l| &l[6..])
                 .ok_or_else(|| {
-                    failure::format_err!(
+                    anyhow::format_err!(
                         "`rustc -vV` didn't have a line for `host:`, got:\n{}",
                         verbose_version
                     )
@@ -250,7 +250,7 @@ fn rustc_fingerprint(path: &Path, rustup_rustc: &Path) -> CargoResult<u64> {
                 .with_extension(env::consts::EXE_EXTENSION);
             paths::mtime(&real_rustc)?.hash(&mut hasher);
         }
-        (true, _, _) => failure::bail!("probably rustup rustc, but without rustup's env vars"),
+        (true, _, _) => anyhow::bail!("probably rustup rustc, but without rustup's env vars"),
         _ => (),
     }
 
