@@ -1,4 +1,6 @@
-use crate::support::{basic_bin_manifest, project};
+//! Tests for `[alias]` config command aliases.
+
+use cargo_test_support::{basic_bin_manifest, project};
 
 #[cargo_test]
 fn alias_incorrect_config_type() {
