@@ -376,8 +376,6 @@ fn explain() {
 #[cargo_test]
 fn z_flags_help() {
     cargo_process("-Z help")
-        .with_stdout_contains(
-            "    -Z unstable-options -- Allow the usage of unstable options such as --registry",
-        )
+        .with_stdout_contains("    -Z unstable-options -- Allow the usage of unstable options")
         .run();
 }
