@@ -1,6 +1,5 @@
 //! Tests for the `cargo metadata` command.
 
-use cargo_test_support::cross_compile::alternate;
 use cargo_test_support::registry::Package;
 use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, main_file, project, rustc_host};
 
@@ -1911,6 +1910,9 @@ fn filter_platform() {
     Package::new("host-dep", "0.0.1").publish();
     Package::new("alt-dep", "0.0.1").publish();
     Package::new("cfg-dep", "0.0.1").publish();
+    // Just needs to be a valid target that is different from host.
+    // Presumably nobody runs these tests on wasm. 🙃
+    let alt_target = "wasm32-unknown-unknown";
     let p = project()
         .file(
             "Cargo.toml",
@@ -1933,7 +1935,7 @@ fn filter_platform() {
                 cfg-dep = "0.0.1"
                 "#,
                 rustc_host(),
-                alternate()
+                alt_target
             ),
         )
         .file("src/lib.rs", "")
@@ -2177,7 +2179,7 @@ fn filter_platform() {
       "links": null
     }
     "#
-    .replace("$ALT_TRIPLE", &alternate())
+    .replace("$ALT_TRIPLE", alt_target)
     .replace("$HOST_TRIPLE", &rustc_host());
 
     // Normal metadata, no filtering, returns *everything*.
@@ -2281,7 +2283,7 @@ fn filter_platform() {
   "workspace_root": "[..]/foo"
 }
 "#
-            .replace("$ALT_TRIPLE", &alternate())
+            .replace("$ALT_TRIPLE", alt_target)
             .replace("$HOST_TRIPLE", &rustc_host())
             .replace("$ALT_DEP", alt_dep)
             .replace("$CFG_DEP", cfg_dep)
@@ -2293,7 +2295,7 @@ fn filter_platform() {
 
     // Filter on alternate, removes cfg and host.
     p.cargo("metadata --filter-platform")
-        .arg(alternate())
+        .arg(alt_target)
         .with_json(
             &r#"
 {
@@ -2355,7 +2357,7 @@ fn filter_platform() {
   "workspace_root": "[..]foo"
 }
 "#
-            .replace("$ALT_TRIPLE", &alternate())
+            .replace("$ALT_TRIPLE", alt_target)
             .replace("$ALT_DEP", alt_dep)
             .replace("$NORMAL_DEP", normal_dep)
             .replace("$FOO", &foo),
@@ -2623,3 +2625,113 @@ fn dep_kinds() {
         )
         .run();
 }
+
+#[cargo_test]
+fn dep_kinds_workspace() {
+    // Check for bug with duplicate dep kinds in a workspace.
+    // If different members select different features for the same package,
+    // they show up multiple times in the resolver `deps`.
+    //
+    // Here:
+    //     foo -> dep
+    //     bar -> foo[feat1] -> dep
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [features]
+            feat1 = []
+
+            [dependencies]
+            dep = { path="dep" }
+
+            [workspace]
+            members = ["bar"]
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "bar/Cargo.toml",
+            r#"
+            [package]
+            name = "bar"
+            version = "0.1.0"
+
+            [dependencies]
+            foo = { path="..", features=["feat1"] }
+            "#,
+        )
+        .file("bar/src/lib.rs", "")
+        .file("dep/Cargo.toml", &basic_lib_manifest("dep"))
+        .file("dep/src/lib.rs", "")
+        .build();
+
+    p.cargo("metadata")
+        .with_json(
+            r#"
+{
+  "packages": "{...}",
+  "workspace_members": "{...}",
+  "target_directory": "[..]/foo/target",
+  "version": 1,
+  "workspace_root": "[..]/foo",
+  "resolve": {
+    "nodes": [
+      {
+        "id": "dep 0.5.0 (path+file://[..]/foo/dep)",
+        "dependencies": [],
+        "deps": [],
+        "features": []
+      },
+      {
+        "id": "bar 0.1.0 (path+file://[..]/foo/bar)",
+        "dependencies": [
+          "foo 0.1.0 (path+file://[..]/foo)"
+        ],
+        "deps": [
+          {
+            "name": "foo",
+            "pkg": "foo 0.1.0 (path+file://[..]/foo)",
+            "dep_kinds": [
+              {
+                "kind": null,
+                "target": null
+              }
+            ]
+          }
+        ],
+        "features": []
+      },
+      {
+        "id": "foo 0.1.0 (path+file://[..]/foo)",
+        "dependencies": [
+          "dep 0.5.0 (path+file://[..]/foo/dep)"
+        ],
+        "deps": [
+          {
+            "name": "dep",
+            "pkg": "dep 0.5.0 (path+file://[..]/foo/dep)",
+            "dep_kinds": [
+              {
+                "kind": null,
+                "target": null
+              }
+            ]
+          }
+        ],
+        "features": [
+          "feat1"
+        ]
+      }
+    ],
+    "root": "foo 0.1.0 (path+file://[..]/foo)"
+  }
+}
+"#,
+        )
+        .run();
+}
