@@ -277,7 +277,7 @@ fn doctest_a_plugin() {
 
 // See #1515
 #[cargo_test]
-fn native_plugin_dependency_with_custom_ar_linker() {
+fn native_plugin_dependency_with_custom_linker() {
     let target = rustc_host();
 
     let _foo = project()
@@ -316,7 +316,6 @@ fn native_plugin_dependency_with_custom_ar_linker() {
             &format!(
                 r#"
             [target.{}]
-            ar = "nonexistent-ar"
             linker = "nonexistent-linker"
         "#,
                 target
@@ -329,7 +328,7 @@ fn native_plugin_dependency_with_custom_ar_linker() {
         .with_stderr_contains(
             "\
 [COMPILING] foo v0.0.1 ([..])
-[RUNNING] `rustc [..] -C ar=nonexistent-ar -C linker=nonexistent-linker [..]`
+[RUNNING] `rustc [..] -C linker=nonexistent-linker [..]`
 [ERROR] [..]linker[..]
 ",
         )
@@ -376,7 +375,7 @@ fn panic_abort_plugins() {
             "bar/src/lib.rs",
             r#"
             #![feature(rustc_private)]
-            extern crate syntax;
+            extern crate rustc_ast;
         "#,
         )
         .build();
@@ -428,7 +427,7 @@ fn shared_panic_abort_plugins() {
             "bar/src/lib.rs",
             r#"
             #![feature(rustc_private)]
-            extern crate syntax;
+            extern crate rustc_ast;
             extern crate baz;
         "#,
         )
