@@ -1,6 +1,8 @@
-use crate::support::{basic_manifest, project};
+//! Tests for the `cargo rustdoc` command.
 
-#[test]
+use cargo_test_support::{basic_manifest, project};
+
+#[cargo_test]
 fn rustdoc_simple() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -8,8 +10,9 @@ fn rustdoc_simple() {
         .with_stderr(
             "\
 [DOCUMENTING] foo v0.0.1 ([CWD])
-[RUNNING] `rustdoc --crate-name foo src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name foo src/lib.rs [..]\
         -o [CWD]/target/doc \
+        [..] \
         -L dependency=[CWD]/target/debug/deps`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
@@ -17,7 +20,7 @@ fn rustdoc_simple() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn rustdoc_args() {
     let p = project().file("src/lib.rs", "").build();
 
@@ -25,8 +28,9 @@ fn rustdoc_args() {
         .with_stderr(
             "\
 [DOCUMENTING] foo v0.0.1 ([CWD])
-[RUNNING] `rustdoc --crate-name foo src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name foo src/lib.rs [..]\
         -o [CWD]/target/doc \
+        [..] \
         --cfg=foo \
         -L dependency=[CWD]/target/debug/deps`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -35,7 +39,7 @@ fn rustdoc_args() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn rustdoc_foo_with_bar_dependency() {
     let foo = project()
         .file(
@@ -64,8 +68,9 @@ fn rustdoc_foo_with_bar_dependency() {
 [CHECKING] bar v0.0.1 ([..])
 [RUNNING] `rustc [..]bar/src/lib.rs [..]`
 [DOCUMENTING] foo v0.0.1 ([CWD])
-[RUNNING] `rustdoc --crate-name foo src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name foo src/lib.rs [..]\
         -o [CWD]/target/doc \
+        [..] \
         --cfg=foo \
         -L dependency=[CWD]/target/debug/deps \
         --extern [..]`
@@ -75,7 +80,7 @@ fn rustdoc_foo_with_bar_dependency() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn rustdoc_only_bar_dependency() {
     let foo = project()
         .file(
@@ -102,8 +107,9 @@ fn rustdoc_only_bar_dependency() {
         .with_stderr(
             "\
 [DOCUMENTING] bar v0.0.1 ([..])
-[RUNNING] `rustdoc --crate-name bar [..]bar/src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name bar [..]bar/src/lib.rs [..]\
         -o [CWD]/target/doc \
+        [..] \
         --cfg=foo \
         -L dependency=[CWD]/target/debug/deps`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -112,7 +118,7 @@ fn rustdoc_only_bar_dependency() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn rustdoc_same_name_documents_lib() {
     let p = project()
         .file("src/main.rs", "fn main() {}")
@@ -123,8 +129,9 @@ fn rustdoc_same_name_documents_lib() {
         .with_stderr(
             "\
 [DOCUMENTING] foo v0.0.1 ([..])
-[RUNNING] `rustdoc --crate-name foo src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name foo src/lib.rs [..]\
         -o [CWD]/target/doc \
+        [..] \
         --cfg=foo \
         -L dependency=[CWD]/target/debug/deps`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
@@ -133,7 +140,7 @@ fn rustdoc_same_name_documents_lib() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn features() {
     let p = project()
         .file(
@@ -156,7 +163,35 @@ fn features() {
         .run();
 }
 
-#[test]
+#[cargo_test]
+fn proc_macro_crate_type() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+
+            [lib]
+            proc-macro = true
+
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("rustdoc --verbose")
+        .with_stderr_contains(
+            "\
+[RUNNING] `rustdoc --crate-type proc-macro [..]`
+",
+        )
+        .run();
+}
+
+#[cargo_test]
 #[cfg(all(target_arch = "x86_64", target_os = "linux", target_env = "gnu"))]
 fn rustdoc_target() {
     let p = project().file("src/lib.rs", "").build();
@@ -165,9 +200,10 @@ fn rustdoc_target() {
         .with_stderr(
             "\
 [DOCUMENTING] foo v0.0.1 ([..])
-[RUNNING] `rustdoc --crate-name foo src/lib.rs [..]\
+[RUNNING] `rustdoc [..]--crate-name foo src/lib.rs [..]\
     --target x86_64-unknown-linux-gnu \
     -o [CWD]/target/x86_64-unknown-linux-gnu/doc \
+    [..] \
     -L dependency=[CWD]/target/x86_64-unknown-linux-gnu/debug/deps \
     -L dependency=[CWD]/target/debug/deps`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]",
